module S4U
    module S4u_toComponents
        @setting_drawline=[]
        class DrawLinetoComponent
            def initialize
                @ip1 = nil
                @ip2 = nil
                @xdown = 0
                @ydown = 0
                @defninstance=nil
                @drawedge=[]
                @ins_draw =true
                @strings=S4U::S4u_toComponents::Strings
            end
            def activate
                model = Sketchup.active_model
                @ip1 = Sketchup::InputPoint.new
                @ip2 = Sketchup::InputPoint.new
                @ip = Sketchup::InputPoint.new
                @setting = S4U::S4u_toComponents::instance_variable_get :@setting_drawline
                if @setting && @setting.length>0
                    @center=@setting[0]
                    @mirror=@setting[1]
                else
                    @center=[1,1]
                    @mirror=[1,1]
                end
                @drawn = false
                @defninstance=self.getinstance
                if !@defninstance
                    Sketchup.active_model.select_tool nil
                    return nil
                end
                Sketchup::set_status_text @strings["Length"], SB_VCB_LABEL
                self.reset_start(nil)
            end
            def getExtents
                bb = Sketchup.active_model.bounds
                bb.add @ip1.position if @ip1.valid?
                bb.add @ip2.position if @ip2.valid?
                bb.add @ip.position if @ip.valid?
                return bb
            end
            def deactivate(view)
                model = Sketchup.active_model
                vars= [@center,@mirror]
                (0..vars.length-1).each{|i| @setting[i]=vars[i] }
                S4U::S4u_toComponents::instance_variable_set(:@setting_drawline,@setting)
                model.start_operation @strings["Draw Line to Components"],true,false,true
                @defninstance.entities.clear! if @ins and @defninstance and @defninstance.valid?
                view.invalidate if @drawn
                model.commit_operation
            end
            def onMouseMove(flags, x, y, view)
                if( @state == 0 )
                    @ip.pick view, x, y
                    if( @ip != @ip1 )
                        view.invalidate if( @ip.display? or @ip1.display? )
                        @ip1.copy! @ip
                        view.tooltip = @ip1.tooltip
                    end
                else
                    @ip2.pick view, x, y, @ip1
                    view.tooltip = @ip2.tooltip if( @ip2.valid? )
                    view.invalidate
                    if( @ip2.valid? )
                        length = @ip1.position.distance(@ip2.position)
                        Sketchup::set_status_text length.to_s, SB_VCB_VALUE
                    end
                end
            end
            def onLButtonDown(flags, x, y, view)
                model = Sketchup.active_model
                if( @state == 0 )
                    @ip1.pick view, x, y
                    if( @ip1.valid? )
                        @state = 1
                        Sketchup::set_status_text @strings["Select second end.Press TAB or Ctrl change insert point.Press Arrow for mirror"], SB_PROMPT
                    end
                elsif ( @ip2.valid? && (@ip1.position!=@ip2.position))
                    self.create_instance(@ip1.position, @ip2.position,view)
                    self.reset(view)
                end
                view.lock_inference
            end
            def onKeyDown(key, repeat, flags, view)
                if key == CONSTRAIN_MODIFIER_KEY
                    if( view.inference_locked? )
                        view.lock_inference
                    elsif( @state == 0 && @ip1.valid? )
                        view.lock_inference @ip1
                    elsif( @state == 1 && @ip2.valid? )
                        view.lock_inference @ip2, @ip1
                    end
                end
            end
            def onKeyUp(key, repeat, flags, view)
                if key == CONSTRAIN_MODIFIER_KEY && view.inference_locked?
                    view.lock_inference
                end
                if ((key == VK_CONTROL) || ((IS_OSX) && (key==VK_ALT)))
                    if @center==[ 1,0]
                        @center=[ 0,1]
                    elsif    @center==[0,1]
                        @center=[ -1,0]
                    elsif    @center==[-1,0]
                        @center=[0,-1]
                    elsif @center==[0,-1]
                        @center=[1,0]
                    else
                        @center=[1,0]
                    end
                end
                if ((key==9)) || ((key==48) && IS_OSX)
                    if @center==[0,0]
                        @center=[-1,-1]
                    elsif  @center==[-1,-1]
                        @center=[-1,1]
                    elsif  @center==[ -1,1]
                        @center=[1,1]
                    elsif  @center==[ 1,1]
                        @center=[1,-1]
                    elsif  @center==[1,-1]
                        @center=[0,0]
                    else
                    @center=[0,0]
                end
                end
                @mirror[0]= -@mirror[0]  if key == VK_LEFT or key == VK_RIGHT
                @mirror[1]= -@mirror[1]  if key == VK_UP or key ==VK_DOWN
                view.invalidate
            end
            def onUserText(text, view)
                model = Sketchup.active_model
                return if not @state == 1
                return if not @ip2.valid?
                begin
                    value = text.to_l
                rescue
                    UI.beep
                    value = nil
                    Sketchup::set_status_text "", SB_VCB_VALUE
                end
                return if !value
                pt1 = @ip1.position
                vec = @ip2.position - pt1
                if( vec.length == 0.0 )
                    UI.beep
                    return
                end
                vec.length = value
                pt2 = pt1 + vec
                self.create_instance(pt1, pt2, view)
                self.reset(view)
            end
            def draw(view)
                model = Sketchup.active_model
                model.select_tool(nil) if @defninstance.deleted? || (!@defninstance) || !((@defninstance.is_a?Sketchup::ComponentDefinition) )
                if( @ip1.valid? )
                    if( @ip1.display? )
                        @ip1.draw(view)
                        @drawn = true
                    end
                    if( @ip2.valid? )
                        @ip2.draw(view) if( @ip2.display? )
                        view.set_color_from_line(@ip1, @ip2)
                        self.draw_instance(@ip1.position, @ip2.position, view)
                        @drawn = true
                    end
                end
            end
            def onCancel(flag, view)
                self.reset_start(view)
            end
            def reset(view)
                @state = 1
                Sketchup::set_status_text @strings["Select next end"], SB_PROMPT
                @ip1.clear
                @ip1.copy! @ip2
                @ip2.clear
                if( view )
                    view.tooltip = nil
                    view.invalidate if @drawn
                end
                #@drawn = false
                #@dragging = false
            end
            def reset_start(view)
                @state = 0
                Sketchup::set_status_text @strings["Select first end"], SB_PROMPT
                @ip1.clear
                @ip2.clear
                if( view )
                    view.tooltip = nil
                    view.invalidate if @drawn
                end
                @drawn = false
                #@dragging = false
            end
            def getinstance
                model = Sketchup.active_model
                ents=model.active_entities
                model.start_operation @strings["Draw Line to Components"],true,false,false
                if not @defninstance
                    sel=model.selection
                    if sel.length==0
                        UI.messagebox @strings["Select a Component Instance or a Face."]
                        model.abort_operation
                        return nil
                    end
                    @ins=group=tmpgrp=ins1=nil
                    flag=0
                    fa=nil
                    ins=sel.grep(Sketchup::ComponentInstance)[0]
                    @ins=ins
                    grp=nil
                    if not ins
                        g=sel.grep(Sketchup::Group)[0]
                        if g
                            grp=g.copy.to_component
                            ins=@ins=grp
                        end
                    end
                    if not ins
                        fa=sel.grep(Sketchup::Face)[0]
                        if fa
                            (sel=sel.to_a-[fa]- fa.edges)
                            flag=1
                        end
                    end
                    if(not ins) and (flag==0)
                        UI.messagebox  @strings["Select a Component Instance or a Face."]
                        model.abort_operation
                        return nil
                    end
                    if flag==1
                        group = ents.add_group
                        tmpgrp=ents.add_group fa
                        new_transform = Geom::Transformation.new(tmpgrp.bounds.center,fa.normal)
                        group.transformation=new_transform
                        inst=group.entities.add_instance(tmpgrp.entities.parent,new_transform.inverse)
                        inst.explode
                        tmpgrp.explode
                        faceingrp=nil
                        group.entities.each{|en|faceingrp = en if en.is_a?(Sketchup::Face)}
                        @unit=group.local_bounds.width if !@unit
                        faceingrp.pushpull (@unit > 0 ? @unit : Sketchup.parse_length("100"))
                        group1=S4U::S4u_toComponents.create_group group,"bottom"
                        group.explode
                        ins1=group1.to_component
                    else
                        gt=ents.add_group ins
                        t=gt.transformation
                        gt.explode
                        inst=ents.add_instance(ins.definition,t)
                        g=S4U::S4u_toComponents.create_group inst,"bottom"
                        ins1=g.to_component
                    end
                    @defninstance=ins1.definition
                    ents.erase_entities ins1
                    ents.erase_entities grp if grp
                end
                model.commit_operation
                return @defninstance
            end
            def linetocom(p1,p2,defn,ents)
                vectoredge= p1.vector_to p2
                cp=Geom::Point3d.new(0,0,0)
                cp.x=@center[0]*( defn.bounds.width / 2)
                cp.y=@center[1]*( defn.bounds.height / 2)
                t1 = Geom::Transformation.new(cp,Geom::Vector3d.new(0,0,1))
                t = Geom::Transformation.new(p1,vectoredge)
                tmpinshtz=p1.distance p2
                inshtz=defn.bounds.depth > 0 ? defn.bounds.depth : 1
                sz= tmpinshtz / inshtz
                sz=1 if sz==0
                s= Geom::Transformation.scaling(@mirror[0],@mirror[1],sz)
                insadd=ents.add_instance defn, t*s*t1
                return insadd
            end
            def create_instance(p1, p2, view)
                model = Sketchup.active_model
                model.start_operation @strings["Draw Line to Components"],true,false,false
                if p1!=p2
                    ents=model.active_entities
                    ins=self.linetocom(p1,p2,@defninstance,ents)
                    ins.explode if @ins
                end
                model.commit_operation
            end
            def get_edge_draw(defn)
                model=Sketchup.active_model
                ents=model.active_entities
                ps=[] ;eds=[]
                model.start_operation @strings["Draw Line to Components"],true,false,true
                ins=ents.add_instance defn,Geom::Transformation.new
                g=ents.add_group ins
                self.explode_all ins
                g.entities.each { |e| eds.push(e) if e.is_a?Sketchup::Edge }
                tg=g.transformation
                eds.each{ |i| ps.push [(i.start.position.transform tg),(i.end.position.transform tg)]}
                ents.erase_entities g
                model.commit_operation
                return ps
            end
            def draw_instance(p1, p2, view)
                if p1!=p2
                    model=Sketchup.active_model
                    ents= model.active_entities
                    defn=@defninstance
                    vectoredge= p1.vector_to p2
                    cp=Geom::Point3d.new(0,0,0)
                    cp.x=@center[0]*( defn.bounds.width / 2)
                    cp.y=@center[1]*( defn.bounds.height / 2)
                    t1 = Geom::Transformation.new(cp,Geom::Vector3d.new(0,0,1))
                    t = Geom::Transformation.new(p1,vectoredge)
                    tmpinshtz=p1.distance p2
                    inshtz=defn.bounds.depth > 0 ? defn.bounds.depth : 1
                    sz= tmpinshtz / inshtz
                    sz=1 if sz==0
                    sz= tmpinshtz / inshtz
                    sz=1 if sz==0
                    s= Geom::Transformation.scaling(@mirror[0],@mirror[1],sz)
                    view.line_width=3
                    view.drawing_color="blueviolet"
                    if @ins_draw
                    @drawedge=self.get_edge_draw(@defninstance)
                    @ins_draw =false
                    end
                    (0..@drawedge.length-1).each{|i|
                        p=@drawedge[i]
                        po1=p[0]; po2=p[1]
                        p1=po1.transform t*s*t1; p2=po2.transform t*s*t1
                        view.draw_line p1,p2
                    }
                end
            end
            def explode_all(com)
                entities=com.explode
                entities.each{ |e| self.explode_all(e) if((e.is_a?Sketchup::Group) ||(e.is_a?Sketchup::ComponentInstance)) } if entities
            end
        end # class LineTool
        def self.drawlinetocomponents
            model = Sketchup.active_model
            Sketchup.active_model.select_tool S4U::S4u_toComponents::DrawLinetoComponent.new
        end
    end
end # Module